@@ -9,7 +9,7 @@ import requests
 # for URL format details.
 
 URL = ('https://eutils.ncbi.nlm.nih.gov/entrez/eutils/efetch.fcgi?'
-       'db=%s&id=%s&rettype=fasta&retmode=text')
+       'db=%s&id=%s&rettype=%s&retmode=text')
 
 parser = argparse.ArgumentParser(
     formatter_class=argparse.ArgumentDefaultsHelpFormatter,
@@ -23,8 +23,12 @@ parser.add_argument(
     '--database', default='nucleotide', choices=('nucleotide', 'protein'),
     help='The name of the NCBI database to query.')
 
+parser.add_argument(
+    '--format', default='fasta', choices=('fasta', 'gb'),
+    help='The format to return.')
+
 args = parser.parse_args()
 
 print(requests.get(
-    URL % (args.database, args.id)
+    URL % (args.database, args.id, args.format)
 ).text.rstrip('\n'))
