@@ -1,17 +1,20 @@
 import sys
 import six
-from os import unlink
+import os
 from functools import total_ordering
 from collections import Counter
 from hashlib import md5
 from random import uniform
+from pathlib import Path
+import itertools
 
 from Bio.Seq import translate
 from Bio.Data.IUPACData import (
     ambiguous_dna_complement, ambiguous_rna_complement)
 
 from dark.aa import (
-    AA_LETTERS, NAMES as AA_NAMES, PROPERTIES, PROPERTY_DETAILS, NONE)
+    AA_LETTERS, NAMES as AA_NAMES, PROPERTIES, PROPERTY_DETAILS, NONE,
+    START_CODON, STOP_CODONS)
 from dark.filter import TitleFilter
 from dark.dna import FloatBaseCounts, AMBIGUOUS, BASES_TO_AMBIGUOUS
 from dark.errors import ReadLengthsNotIdenticalError
@@ -117,7 +120,7 @@ class Read(object):
         quality = None if self.quality is None else self.quality[item]
         return self.__class__(self.id, sequence, quality)
 
-    def toString(self, format_):
+    def toString(self, format_='fasta'):
         """
         Convert the read to a string format.
 
@@ -340,6 +343,56 @@ class DNARead(_NucleotideRead):
 
     COMPLEMENT_TABLE = _makeComplementTable(ambiguous_dna_complement)
 
+    def findORF(self, offset: int, forward: bool = True,
+                requireStartCodon: bool = True):
+        """
+        Find an ORF that supposedly starts at a specified offset in a read.
+
+        @param offset: The C{int} offset of the start codon.
+        @param forward: If not C{True}, the reverse complement of the sequence
+            should be examined.
+        @param requireStartCodon: If C{True}, the first codon must be a start
+            codon. If it is not, the search is abandoned immediately and the
+            returned dictionary will have zero and C{False} values.
+        @return: A C{dict} with C{str} keys:
+            length (int): the length of the ORF
+            foundStartCodon (bool): if a start codon was found.
+            foundStopCodon (bool): if a stop codon was found.
+        """
+        sequence = (
+            self.sequence if forward else self.reverseComplement().sequence)
+
+        first = True
+        length = 0
+        foundStartCodon = foundStopCodon = False
+        codons = []
+
+        for index in itertools.count(offset, 3):
+            codon = sequence[index:index + 3]
+            if len(codon) != 3:
+                break
+
+            if first:
+                first = False
+                if codon == START_CODON:
+                    foundStartCodon = True
+                elif requireStartCodon:
+                    break
+
+            length += 1
+            codons.append(codon)
+
+            if codon in STOP_CODONS:
+                foundStopCodon = True
+                break
+
+        return {
+            'length': length,
+            'foundStartCodon': foundStartCodon,
+            'foundStopCodon': foundStopCodon,
+            'translation': translate(''.join(codons)),
+        }
+
 
 class RNARead(_NucleotideRead):
     """
@@ -965,14 +1018,20 @@ class ReadFilter(object):
         self.maxNFraction = maxNFraction
         self.removeGaps = removeGaps
         self.head = head
-        self.removeDuplicates = removeDuplicates
-        self.removeDuplicatesById = removeDuplicatesById
-        self.removeDuplicatesUseMD5 = removeDuplicatesUseMD5
         self.removeDescriptions = removeDescriptions
         self.modifier = modifier
         self.randomSubset = randomSubset
         self.trueLength = trueLength
 
+        if removeDuplicatesUseMD5 and not (
+                removeDuplicates or removeDuplicatesById):
+            raise ValueError(
+                'If you specify removeDuplicatesUseMD5, you need to also use '
+                'one of removeDuplicates or removeDuplicatesById.')
+        self.removeDuplicates = removeDuplicates
+        self.removeDuplicatesById = removeDuplicatesById
+        self.removeDuplicatesUseMD5 = removeDuplicatesUseMD5
+
         if keepSequences and removeSequences:
             raise ValueError(
                 'Cannot simultaneously filter using keepSequences and '
@@ -1381,14 +1440,14 @@ class Reads(object):
         format_ = format_.lower()
         count = 0
 
-        if isinstance(filename, str):
+        if isinstance(filename, (str, Path)):
             try:
                 with open(filename, 'w') as fp:
                     for read in self:
                         fp.write(read.toString(format_))
                         count += 1
             except ValueError:
-                unlink(filename)
+                os.unlink(filename)
                 raise
         else:
             # We have a file-like object.
