@@ -1,8 +1,8 @@
-import six
 import bz2
 import gzip
 from six.moves import builtins
 from unittest import TestCase
+from unittest.mock import mock_open
 from six import assertRaisesRegex
 from collections import Counter
 
@@ -11,12 +11,12 @@ try:
 except ImportError:
     from mock import patch
 
-from .mocking import mockOpen
+from io import BytesIO
 
 from dark.utils import (
     numericallySortFilenames, median, asHandle, parseRangeString,
     parseRangeExpression, pct, StringIO, baseCountsToStr, nucleotidesToStr,
-    countPrint)
+    countPrint, take, matchOffset)
 
 
 class TestNumericallySortFilenames(TestCase):
@@ -129,8 +129,7 @@ class TestAsHandle(TestCase):
         When an open file pointer is passed to asHandle, that same file
         pointer must be returned.
         """
-        mockOpener = mockOpen()
-        with patch.object(builtins, 'open', mockOpener):
+        with patch.object(builtins, 'open', mock_open()):
             fp = open('file')
             with asHandle(fp) as newfp:
                 self.assertIs(fp, newfp)
@@ -140,7 +139,7 @@ class TestAsHandle(TestCase):
         When a string filename is passed to asHandle, it must be possible to
         read the correct data from the fp that is returned.
         """
-        mockOpener = mockOpen(read_data='xxx')
+        mockOpener = mock_open(read_data='xxx')
         with patch.object(builtins, 'open', mockOpener):
             with asHandle('file') as fp:
                 self.assertEqual('xxx', fp.read())
@@ -150,13 +149,7 @@ class TestAsHandle(TestCase):
         When a string '*.bz2' filename is passed to asHandle, it must be
         possible to read the correct data from the fp that is returned.
         """
-        if six.PY3:
-            self.skipTest('Mocking bz2.BZ2File disabled under Python 3')
-
-        # This test should be better. It should actually create some bz2
-        # compressed data and make sure that it's decompressed
-        # properly. But Python mocking makes me so confused...
-        result = StringIO('xxx')
+        result = BytesIO(b'xxx')
 
         with patch.object(bz2, 'BZ2File') as mockMethod:
             mockMethod.return_value = result
@@ -168,13 +161,7 @@ class TestAsHandle(TestCase):
         When a string '*.gz' filename is passed to asHandle, it must be
         possible to read the correct data from the fp that is returned.
         """
-        if six.PY3:
-            self.skipTest('Mocking gzip.GzipFile disabled under Python 3')
-
-        # This test should be better. It should actually create some gzip
-        # compressed data and make sure that it's decompressed
-        # properly. But Python mocking makes me so confused...
-        result = StringIO('xxx')
+        result = BytesIO(b'xxx')
 
         with patch.object(gzip, 'GzipFile') as mockMethod:
             mockMethod.return_value = result
@@ -512,3 +499,109 @@ class TestPct(TestCase):
         one seventh of the denominator.
         """
         self.assertEqual('2/14 (14.286%)', pct(2, 14))
+
+
+class TestTake(TestCase):
+    """
+    Test the take function.
+    """
+    def testNLessThanOne(self):
+        """
+        The take function must raise an AssertionError if passed n < 1.
+        list.
+        """
+        self.assertRaises(AssertionError, next, take([], 0))
+
+    def testEmpty(self):
+        """
+        The take function must return an empty generator if passed an empty
+        list.
+        """
+        self.assertEqual([], list(take([], 4)))
+
+    def testOne(self):
+        """
+        The take function must return individual items if passed n=1.
+        """
+        self.assertEqual([[3], [4], [5]],
+                         list(take([3, 4, 5], 1)))
+
+    def testNBiggerThanPassedList(self):
+        """
+        The take function must return all items if passed an n that is bigger
+        than the passed list.
+        """
+        self.assertEqual([[3, 4, 5]],
+                         list(take([3, 4, 5], 100)))
+
+    def testThree(self):
+        """
+        The take function must return three items at a time if passed n=3, and
+        also return the extra two items.
+        """
+        self.assertEqual([[0, 1, 2], [3, 4, 5], [6, 7, 8], [9, 10]],
+                         list(take(range(11), 3)))
+
+
+class TestMatchOffset(TestCase):
+    """
+    Test the matchOffset function.
+    """
+    def testEmpty(self):
+        """
+        An empty query must match an empty reference at position 0.
+        """
+        self.assertEqual(0, matchOffset('', ''))
+
+    def testEqualStrings(self):
+        """
+        An non-empty reference must match an identical non-empty query at
+        position 0.
+        """
+        self.assertEqual(0, matchOffset('AA', 'AA'))
+
+    def testQueryPaddedLeftByOne(self):
+        """
+        A query that is padded on the left by one space must match at
+        position 1.
+        """
+        self.assertEqual(1, matchOffset('AA', ' A'))
+
+    def testQueryPaddedLeftByTwo(self):
+        """
+        A query that is padded on the left by two spaces must match at
+        position 2.
+        """
+        self.assertEqual(2, matchOffset('AAA', '  A'))
+
+    def testQueryPaddedLeftByTwoReferencePaddedLeftByOne(self):
+        """
+        A query that is padded on the left by two spaces must match a reference
+        that is padded on the left by one space at position 1.
+        """
+        self.assertEqual(1, matchOffset(' AA', '  A'))
+
+    def testQueryPaddedLeftByFiveReferencePaddedLeftByOneWithGaps(self):
+        """
+        A query that is padded on the left by five spaces must correctly match
+        a reference containing gaps that is padded on the left by one space.
+        """
+        self.assertEqual(2, matchOffset(' AA--G',
+                                        '     A'))
+
+    def testQueryPaddedLeftByFiveReferenceOneGapLeftWithGaps(self):
+        """
+        A query that is padded on the left by five spaces must correctly match
+        a reference that starts with a gap and that contains gaps.
+        """
+        self.assertEqual(2, matchOffset('-AA--G',
+                                        '     A'))
+
+    def testOmicronPartialInsertionRead(self):
+        """
+        A query that overlaps part of an insertion in the reference must be
+        handled correctly.
+        """
+        self.assertEqual(12, matchOffset(
+            'TAATTTAGTGCG---------TGATCTCCCTCAGGGTTTTTCGGCTTTAGAAC',
+            '             AGCCAGAATGATCTCCCTCAGGGTTTTTCGGCTTT'))
